@@ -7,17 +7,17 @@ from astropy.tests.pytest_plugins import pytest_addoption as astropy_pytest_addo
 
 # Uncomment the following line to treat all DeprecationWarnings as
 # exceptions
-# enable_deprecations_as_exceptions()
+enable_deprecations_as_exceptions()
 
 import os
 from astropy.tests.helper import pytest
 
 
 def pytest_addoption(parser):
-    parser.addoption('--generate-reference', help="generate reference images", action="store_true")
+    parser.addoption('--generate-images-path', help="directory to generate reference images in", action='store')
     return astropy_pytest_addoption(parser)
 
 
 @pytest.fixture
 def generate(request):
-    return request.config.getoption("--generate-reference")
+    return request.config.getoption("--generate-images-path")
